﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace REG_MARK_LIB
{
    public static class REG_MARK
    {
        /// <summary>
        /// проверяет переданный номерной знак в формате a999aa999
        /// </summary>
        /// <param name="mark">номерной знак в виде строки (латинские буквы)</param>
        /// <returns>true-номерной знак написан верно, false-номерной знак написон не верно</returns>
        public static Boolean CheckMark(String mark)
        {
            try
            {
                string pattern = @"^[a-zA-z]\d{3}[a-zA-z]{2}\d{3}$";
                Regex regex = new Regex(pattern);
                if (regex.IsMatch(mark))
                {
                    string number = mark.Substring(6);
                    int regionNumber;
                    if (int.TryParse(number, out regionNumber))
                    {
                        if (regionNumber >= 1 && regionNumber <= 999)
                        {
                            return true;
                        }
                    }
                }
                return false;
            }
            catch (Exception)
            {

                return false;
            }
           
        }

        /// <summary>
        /// выдает следующий номер в данной серии или создает следующую серию
        /// </summary>
        /// <param name="mark">номерной знак в виде строки (латинские буквы)</param>
        /// <returns></returns>
        /// <exception cref="Exception">невозможно создать следующую серию</exception>
        public static String GetNextMarkAfter(String mark)
        {
            try
            {
                char[] marcChars = mark.ToCharArray();
                int number = int.Parse(new string(marcChars, 1, 3)) + 1;
                int numberRegionInt = int.Parse(new string(marcChars, 6, 3));
                string newNumber = number.ToString().PadLeft(3, '0');
                string NumberRegion = numberRegionInt.ToString().PadLeft(3, '0');
                if (number > 999)
                {
                    char firstChar = marcChars[0];
                    char middleChar = marcChars[4];
                    char lastChar = marcChars[5];
                    firstChar++;
                    if (firstChar >= 'z')
                    {
                        firstChar = 'a';
                        lastChar++;
                        if (lastChar > 'z')
                        {
                            throw new Exception();
                            
                        }
                    }
                    return $"{firstChar}{newNumber}{middleChar}{lastChar}{NumberRegion}";
                }
                else
                {
                    return $"{marcChars[0]}{newNumber}{marcChars[4]}{marcChars[5]}{NumberRegion}";
                }
            }
            catch (Exception)
            {

                return "s";
            }
            
        }

        /// <summary>
        /// выдает следующий номер в промежутке
        /// </summary>
        /// <param name="prevMark">номерной знак</param>
        /// <param name="rangeStart">начало промежутка</param>
        /// <param name="rangeEnd">конец промежутка</param>
        /// <returns>выдать номер, в противном случае “out of stock”</returns>
        public static String GetNextMarkAfterInRange(String prevMark, String rangeStart, String rangeEnd)
        {
            try
            {
                if (string.Compare(prevMark, rangeEnd) > 0 || string.Compare(prevMark, rangeStart) < 0)
                {
                    return "out of stock";
                }
                char[] marcChars = prevMark.ToCharArray();
                int number = int.Parse(new string(marcChars, 1, 3)) + 1;
                int numberRegionInt = int.Parse(new string(marcChars, 6, 3));
                string newNumber = number.ToString().PadLeft(3, '0');
                string NumberRegion = numberRegionInt.ToString().PadLeft(3, '0');
                if (number > 999)
                {
                    char firstChar = marcChars[0];
                    char middleChar = marcChars[4];
                    char lastChar = marcChars[5];
                    firstChar++;
                    if (lastChar > 'z')
                    {
                        return "out of stock";
                    }
                    string nextMark = $"{firstChar}{newNumber}{middleChar}{lastChar}{NumberRegion}";
                    if (string.Compare(nextMark, rangeEnd) > 0)
                    {
                        return "out of stock";
                    }
                    return nextMark;
                }
                else
                {
                    string nextMark = $"{marcChars[0]}{newNumber}{marcChars[4]}{marcChars[5]}{NumberRegion}";
                    if (string.Compare(nextMark, rangeEnd) > 0)
                    {
                        return "out of stock";
                    }
                    return nextMark;
                }
            }
            catch (Exception)
            {

                return "s";
            }
            
            
        }


        /// <summary>
        /// количество возможных номеров
        /// </summary>
        /// <param name="mark1">начало промежутка</param>
        /// <param name="mark2">конец промежутка</param>
        /// <returns></returns>
        public static int GetCombinationsCountInRange(String mark1, String mark2)
        {
            try
            {
                int count = 0;
                if (string.Compare(mark1, mark2) == 0)
                {
                    return 1;
                }
                char[] mark1Char = mark1.ToCharArray();
                char[] mark2Char = mark2.ToCharArray();

                char firstChar1 = mark1Char[0];
                int number1 = int.Parse(new string(mark1Char, 1, 3));
                char lastChar1 = mark1Char[5];
                char middleChar1 = mark1Char[4];
                int number2 = int.Parse(new string(mark1Char, 6, 3));

                char firstChar2 = mark2Char[0];
                int number3 = int.Parse(new string(mark2Char, 1, 3));
                char lastChar2 = mark2Char[5];
                char middleChar2 = mark1Char[4];
                int number4 = int.Parse(new string(mark1Char, 6, 3));


                count += (firstChar2 - firstChar1);
                count += (number3 - number1);
                count += (number4 - number2);
                count += (lastChar2 - lastChar1);
                count += (middleChar2 - middleChar1);
                return count;
            }
            catch (Exception)
            {

                return 0;
            }
            
        }
    }
}
